function [U,obj] = MMMF(X,gamma,U0,MASK,maxit)
% Input:
%   X is a NxN matrix containing values in {0,1}.
%   MASK gives weight to the losses at each element.
%   gamma is weight decay for the factor U.
%   U0 is initialization for the factor U.
%
% Output:
%   U is the factor of size Nxk.

if any(diag(MASK))
  error('MMMF: The graph shall not have self-loop!\n');
end

if any(MASK(:)<0)
  error('MMMF: The MASK shall contain non-negative weights!\n');
end

if ~exist('maxit','var') || isempty(maxit)
  maxit=100;
end

[N,K]=size(U0);

% First turn X into the sign matrix.
Y=2*X-1;

% Hinge loss at each element.
% loss=max( 1 - Y.*(U*U'), 0 );
% e = sum(sum( MASK.*loss )) + gamma*sum(sum(U0.^2));

[U,obj]=minimize(U0(:),@errorfunc,maxit,Y,MASK,gamma);
U=reshape(U,N,K);

% % % N=10; Y=randn(N); Y=sign(Y+Y')
% % % MASK=rand(N); MASK=(MASK+MASK')/2; MASK(MASK<0.5)=0;
% % % gamma=0.3;
% % % U0=randn(N,5);
% % % derivativeCheck(@errorfunc,U0(:),1,2,Y,MASK,gamma)
