function [e,grad]=errorfunc(U,Y,MASK,gamma)
% Error function handler.

N=size(Y,1); K=numel(U)/N;
U=reshape(U,N,K);

loss=max( 1 - Y.*(U*U'), 0 );
maskedloss=MASK.*loss;
e = sum(sum( maskedloss )) + gamma*sum(sum(U.^2));

if nargout>1
  % Find indices with nonzero loss.
  IND=find(maskedloss);  M=length(IND);
  [I,J]=ind2sub([N,N],IND);
  
  tmpidx1=repmat( J, 1, K); tmpidx2=repmat( 1:K, M, 1 );
  tmpval= -spdiags(MASK(IND).*Y(IND),0,M,M)*U(I,:);
  grad=accumarray( [tmpidx1(:) tmpidx2(:)], tmpval(:), [N,K]);
  tmpidx1=repmat( I, 1, K); % tmpidx2 does not change.
  tmpval= -spdiags(MASK(IND).*Y(IND),0,M,M)*U(J,:);
  grad=grad+accumarray( [tmpidx1(:) tmpidx2(:)], tmpval(:), [N,K]);
  grad=grad+2*gamma*U;
  grad=grad(:);
end




