# include "mex.h"
# include <queue>
# include <vector>
# include <string>
# include <algorithm>
# include <ctime>
# include <fstream>
# include <sstream>
# include <cmath>
# include <cstdlib>


# define DD_STEPSIZE 2
# define DD_ITER 30
# define DD_EPS 0.5e-2
# define DD_CHANGE 1
# define PMAX 20000


using namespace std;

struct RANK_SCORE
{
  double value;
  int index;
};

struct FOR_RANK
{ 
  int a;
  int b;
  double c;
};

bool mycmpcmp(const FOR_RANK &x, const FOR_RANK &y)
{ 
  return fabs(x.c) > fabs(y.c);
}

bool cmp_score(const RANK_SCORE &a, const RANK_SCORE &b)
{
  return (a.value > b.value);
}

bool cmp_absolute_score(const RANK_SCORE &a, const RANK_SCORE &b)
{
  return (fabs(a.value) > fabs(b.value));
}

void GENP(int n, double gamma, int *g, int recall)
{
  double *STORE;
  double sum = 0;
  double nodesum = 0;
  int index = 1;
  int mysize;

  STORE = new double [PMAX+1];

  if (gamma > 2.1)
    {
      mysize = PMAX;
    }
  else
    {
      mysize = n;
    }

  for (int i=1; i<=mysize; ++i)
    {
      STORE[i] = pow(1.0/i,gamma);
    }
  for (int i=mysize; i>0; --i)
    {
      sum += STORE[i];
    }

  for (int i=mysize; i>0; --i)
    {

      nodesum += STORE[i];

      while (nodesum >= sum/n)
        {
          g[index] = i;
          ++index;
          nodesum -= sum/n;
        }

      if (index > n)
        {
          break;
        }
    }

  if (nodesum > 0)
    {
      g[n] = 1;
    }

  sum = 0;
  for (int i=1; i<n+1; ++i)
    {
      sum += g[i];
    }
  for (int i=1; i<n+1; ++i)
    {
      g[i] = 0.5 + 2.0*recall*g[i]/sum;
      if (g[i] > n-1)
	{
	  g[i] = n-1;
	}
    }

  delete []STORE;
}

void GENH(int n, double *h)
{
  for (int i=1; i<n; ++i)
    {
      h[i] = log(i+1)-log(i);
    }
}

void TH(int n, double *th, int recall)
{
  for (int i=1; i<n+1; ++i)
    {
      th[i] = pow(log(i+1),0.25);
    }
}

void TG(int n, int *g, double *th, double *tg)
{
  for (int i=1; i<=n; ++i)
    {
      if (g[i] != 0)
	{
	  tg[n+1-i] = 1.0/th[g[i]];
	}
      else
	{
	  tg[n+1-i] = 1.1/th[1];
	}
    }
}

void CAL_SCORE(int n, double **M, double *h, struct RANK_SCORE *sort_score)
{
  priority_queue <double> pq;

  for (int i=1; i<n+1; ++i)
    {
      for (int j=1; j<n+1; ++j)
        {
          if (i != j)
            {
              pq.push(fabs(M[i][j]));
            }
        }

      sort_score[i].value = 0;
      sort_score[i].index = i;

      for (int j=1; j<n; ++j)
        {
          sort_score[i].value += pq.top()*h[j];
          pq.pop();
        }
    }
}

void ROW_SCORE(int n, int id, double *myans, double *M, double *h, double rho, double alpha, struct RANK_SCORE *myscore, double *old_h)
{
  alpha /= rho;

  int len;
  int *sumlen;
  double *mysum;
  double *ans;
  struct RANK_SCORE *myM;

  sumlen = new int[n+2];
  mysum = new double[n+2];
  ans = new double[n+2];
  myM = new RANK_SCORE[n+2];

  sumlen[0] = 0;
  mysum[0] = 0;
  len = 0;
  for (int i=1; i<id; ++i)
    { 
      myM[i].value = M[i];
      myM[i].index = i;
    }

  for (int i=id+1; i<n+1; ++i)
    { 
      myM[i-1].value = M[i];
      myM[i-1].index = i;
    }

  sort(myM+1,myM+n,cmp_absolute_score);

  for (int i=1; i<n; ++i)
    {
      mysum[i] = mysum[i-1] + fabs(myM[i].value)-h[i]*alpha;
      int left = 1;
      int right = len;
      int mid;

      while (right >= left)
        {
          mid = (left+right)/2;

          double tmp = ans[mid]*(i-sumlen[mid-1])-(-mysum[sumlen[mid-1]]+mysum[i]);

          if (tmp == 0 || ans[mid] == 0)
            {
              break;
            }
          else if (tmp > 0)
            {
              left = mid+1;
            }
          else
            {
              right = mid-1;
            }
        }

      if (left > right)
        {
          len = right+1;
          sumlen[len] = i;
          ans[len] = (mysum[i]-mysum[sumlen[len-1]]>0)?(mysum[i]-mysum[sumlen[len-1]]):0;
          ans[len] /= (i-sumlen[len-1]);
        }
      else
        {
          len = mid;
          ans[mid] = (mysum[i]-mysum[sumlen[mid-1]]>0)?(mysum[i]-mysum[sumlen[mid-1]]):0;
          sumlen[mid] = i;
          ans[mid] /= (i-sumlen[mid-1]);
        }
    }

  myscore[id].value = 0;
  myscore[id].index = id;
  
  for (int i=1; i<=len; ++i)
    {
      for (int j=sumlen[i-1]+1; j<=sumlen[i]; ++j)
        {
          if (myM[j].value > 0)
            {
              myans[myM[j].index] = ans[i];
            }
          else
            {
              myans[myM[j].index] = 0-ans[i];
            }

          myscore[id].value += ans[i]*old_h[j];
        }
    }

  myans[id] = M[id];

  delete []sumlen;
  delete []mysum;
  delete []myM;
  delete []ans;
}

void NON_SYMMETRIC_OPT(int n, double **myans, double **M, double *h, double *th, double *g, double rho, double alpha, double beta, struct RANK_SCORE *myscore)
{
  double **tmph;
  tmph = new double *[n+1];
  for (int i=0; i<n+1; ++i)
    {
      tmph[i] = new double [n+1];
    }

  for (int i=1; i<n+1; ++i)
    {
      for (int j=1; j<n; ++j)
        {
          tmph[i][j] = alpha*h[j] + beta*g[i]*th[j];
        }

      ROW_SCORE(n,i,myans[i],M[i],tmph[i],rho,1,myscore,th);
    }

  for (int i=0; i<n+1; ++i)
    {
      delete [] tmph[i];
    }
  delete []tmph;
}

void DD(int n, double **myans, double **M, double *ph, double *h, double *g, double rho, double alpha, double theta, struct RANK_SCORE *myscore)
{
  int **check_sign;
  double **step_size;
  double **beta;
  bool break_sign;

  srand(time(NULL));

  check_sign = new int *[n+1];
  step_size = new double *[n+1];
  beta = new double *[n+1];
  for (int i=0; i<n+1; ++i)
    {
      beta[i] = new double [n+1];
      step_size[i] = new double [n+1];
      check_sign[i] = new int [n+1];
    }

  for (int i=0; i<n+1; ++i)
    {
      for (int j=0; j<n+1; ++j)
        {
          beta[i][j] = M[i][j];
          step_size[i][j] = DD_STEPSIZE;
          check_sign[i][j] = 0;
        }
    }

  for (int i=0; i<DD_ITER; ++i)
    {
      break_sign = true;

      NON_SYMMETRIC_OPT(n, myans, beta, ph, h, g, rho, alpha, theta, myscore);

      for (int j=1; j<n+1; ++j)
        {
          for (int k=j; k<n+1; ++k)
            {
              if (fabs(myans[j][k]-myans[k][j]) > DD_EPS*(fabs(myans[j][k])+fabs(myans[k][j])))
                {
                  break_sign = false;
                }

              if ((myans[j][k]-myans[k][j])*check_sign[j][k] > 0)
                {
                  step_size[j][k] = step_size[j][k]*(2*DD_CHANGE*(1.0+0.01*(rand()%100)));
                }
              else if ((myans[j][k]-myans[k][j])*check_sign[j][k] < 0)
                {
                  step_size[j][k] = step_size[j][k]/(2*DD_CHANGE*(1.0+0.01*(rand()%100)));
                }

              if (myans[j][k] - myans[k][j] > 0)
                {
                  check_sign[j][k] = 1;
                }
              else if (myans[j][k] - myans[k][j] < 0)
                {
                  check_sign[j][k] = -1;
                }
              else
                { 
                  check_sign[j][k] = 0;
                }

              beta[j][k] -= 2*step_size[j][k]/rho*(myans[j][k]-myans[k][j]);
              beta[k][j] += 2*step_size[j][k]/rho*(myans[j][k]-myans[k][j]);
            }
        }

      if (break_sign)
        {
          break;
        }
    }

  for (int i=1; i<n+1; ++i)
    {
      for (int j=i; j<n+1; ++j)
        {
          myans[i][j] = myans[j][i] = (myans[i][j]+myans[j][i])/2;
        }
    }

  for (int i=0; i<n+1; ++i)
    {
      delete [] check_sign[i];
      delete [] step_size[i];
      delete [] beta[i];
    }
  delete []check_sign;
  delete []step_size;
  delete []beta;
}

void ADJUST(int n, double **myans, double **M, double *h, double *th, double *g, double rho, double alpha, double theta)
{
  double asize = 0.5;
  int alimit = 50;
  srand(time(NULL));

  struct RANK_SCORE *sort_score_1;
  struct RANK_SCORE *sort_score_2;

  double *beta;
  double *penaltyg;
  double *nodeg;

  double *mysize;
  double *mydiff;

  sort_score_1 = new struct RANK_SCORE [n+2];
  sort_score_2 = new struct RANK_SCORE [n+2];

  penaltyg = new double [n+2];
  nodeg = new double [n+2];
  beta = new double [n+2];
  mysize = new double [n+2];
  mydiff = new double [n+2];

  for (int i=1; i<n+1; ++i)
    { 
      beta[i] = 0;
      mydiff[i] = 0;
      mysize[i] = asize;
    }

  CAL_SCORE(n, M, h, sort_score_1);
  sort(sort_score_1+1, sort_score_1+n+1, cmp_score);
  for (int i=1; i<n+1; ++i)
    { 
      penaltyg[sort_score_1[i].index] = g[n-i+1];
    }

  bool pre_sign = false;
  bool curr_sign;
 
  for (int iter = 0; iter < alimit; ++iter)
    {
      curr_sign = true;
 
      DD(n, myans, M, h, th, penaltyg, rho, alpha, theta, sort_score_1);
  
      for (int i=1; i<n+1; ++i)
        { 
          sort_score_1[i].value += beta[i];
        }
      sort(sort_score_1+1, sort_score_1+n+1, cmp_score);
      for (int i=1; i<n+1; ++i)
        { 
          penaltyg[sort_score_1[i].index] = g[n-i+1];
        }
      CAL_SCORE(n, myans, h, sort_score_2);
      sort(sort_score_2+1, sort_score_2+n+1, cmp_score);
      
      for (int i=1; i<n+1; ++i)
        {
          nodeg[sort_score_2[i].index] = g[n-i+1];
        }
      for (int i=1; i<n+1; ++i)
        { 
          if ((penaltyg[i] - nodeg[i])*mydiff[i] > 0)
            { 
              mysize[i] *= (1.3*(1.0+0.005*(rand()%100)));
            }
          else if ((penaltyg[i] - nodeg[i])*mydiff[i] < 0)
            { 
              mysize[i] /= (1.0+0.005*(rand()%100));
            }

          beta[i] += mysize[i]*(penaltyg[i]-nodeg[i]);
          mydiff[i] = penaltyg[i] - nodeg[i];

          if (penaltyg[i] != nodeg[i])
            { 
              curr_sign = false;
            }
        }

      if (curr_sign && pre_sign)
        {
          break;
        }

      // Just rank top nodes
   
      curr_sign = true;
      int tmp=n*0.5;
      //if (tmp>300)
        //  tmp=300;
      for (int i=1; i<tmp; ++i)
	{
	  if (penaltyg[sort_score_1[i].index] != nodeg[sort_score_2[i].index])
	    {
	      curr_sign = false;
	      break;
	    }
	}

      if (curr_sign && pre_sign)
	{
	  break;
	}

      // Just rank top nodes

      pre_sign = curr_sign;
    }
 
  delete []sort_score_1;
  delete []sort_score_2;
  delete []beta;
  delete []penaltyg;
  delete []nodeg;
  delete []mysize;
  delete []mydiff;
}

void Lovasz(double *M, double *Z, int n, double gamma, double rho, double beta, int recall,double *ans)
{
		
	double *h;
	int *g;
	double *tg;
	double *th;
	double alpha=0;
	bool symmetric = false;
	double **re_M;
    double **re_ans;
    re_M = new double*[n+1];
    re_ans = new double*[n+1];

   for (int i=0; i<n+1; ++i)
    {
      re_ans[i] = new double[n+1];
      re_M[i] = new double[n+1];
    }
    
  for (int i=0; i<n+1; ++i)
    {
      re_M[i][0] = 0;
      re_M[0][i] = 0;
      re_ans[i][0] = 0;
      re_ans[i][0] = 0;
      
    }
    
	h = new double[n+1];
	g = new int[n+1];
	th = new double[n+1];
	tg = new double[n+1];
	
  
	GENH(n,h);
	GENP(n,gamma,g,recall);
	TH(n,th,recall);
	TG(n,g,th,tg);

	for (int i=0; i<n; ++i)
		for (int j=0; j<n; ++j){
             re_M[i+1][j+1] = M[i*n+j]+Z[i*n+j];
             if (re_M[i+1][j+1]<0)
                re_M[i+1][j+1]=0;
	}
    
	
	ADJUST(n, re_ans, re_M, h, th, tg, rho, alpha, beta);
		

   for (int i=0; i<n; ++i)
    {
      for (int j=0; j<n; ++j)
	{
	  ans[i*n+j] = re_ans[i+1][j+1];
	}
   }

   
// Release memory
  for (int i=0; i<n+1; ++i)
    {
      delete [] re_M[i];
      delete [] re_ans[i];
    }
  delete []re_M;
  delete []re_ans;
  delete []h;
  delete []g;
  delete []tg;
  delete []th;

}


void mexFunction ( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] ){
	double *Matrix;
    double *ans;
    double *Z;

    double gamma;
    double rho;
    double beta;
    int n;

    int recall;
 
	Matrix = mxGetPr(prhs[0]);
    Z=mxGetPr(prhs[1]);
	n = mxGetScalar(prhs[2]);
  
	rho = mxGetScalar ( prhs[4] );
    gamma=mxGetScalar ( prhs[3] );
	beta = mxGetScalar ( prhs[5] );
    
	plhs[0] = mxCreateDoubleMatrix ( n*n,1, mxREAL );
	ans= mxGetPr ( plhs[0] );
    recall= mxGetScalar ( prhs[6] );
	Lovasz(Matrix, Z, n, gamma, rho, beta,recall, ans);
	return;

} 