# include "mex.h"
# include <queue>
# include <vector>
# include <string>
# include <algorithm>
# include <iostream>
# include <fstream>
# include <sstream>
# include <cmath>
# include <ctime>
# include <cstdlib>

# define DD_STEPSIZE 1
# define DD_ITER 50
# define DD_EPS 0.5e-3
# define DD_CHANGE 1.2

# define PMAX 10000

using namespace std;

struct RANK_SCORE
{ 
  double value;
  int index;
};

bool cmp_absolute_score(const RANK_SCORE &a, const RANK_SCORE &b)
{
  return (fabs(a.value) > fabs(b.value));
}

/*void GENP(int n, double gamma, int *g)
{ 
  double *STORE;
  double sum = 0;
  double nodesum = 0;
  int index = 1;
  STORE = new double [PMAX+1];
  
  for (int i=1; i<=PMAX; ++i)
    {
      STORE[i] = pow(1.0/i,gamma);
    }
  for (int i=PMAX; i>0; --i)
    {
      sum += STORE[i];
    }

  for (int i=PMAX; i>0; --i)
    {

      nodesum += STORE[i];
      while (nodesum >= sum/n)
        {
          g[index] = i;
          ++index;
          nodesum -= sum/n;
        }
  
      if (index > n)
        {
          break;
        }
    }
  
  if (nodesum > 0)
    {
      g[n] = 1;
    }
  
  delete []STORE;
}
*/

// g[i] is the expected number of true interactions around protein i
void GENPP(int n, double *expected, double *observed, int *g)
{
  for (int i=1; i<=n; ++i)
        g[i] = expected[i];
}

void GENH(int n, double *h)
{ 
  for (int i=1; i<n; ++i)
    {
      h[i] = log(i+1)-log(i);
    }
}

void TH(int n, double *th,double ScaleFree)
{
  for (int i=1; i<n+1; ++i)
    {
      th[i] = pow(log(i+1),ScaleFree);
    }
}

// tg[i] is the node penalty of protein i 
void TG(int n, int *g, double *th, double *tg)
{
  for (int i=1; i<n+1; ++i)
    {
      if (g[i] != 0)
	{ 
	  tg[i] = 1.0/th[g[i]];
	}
      else
	{
	  tg[i] = 1.1/(th[1]);
	}
    }
}

void ROW_SCORE(int n, int id, double *myans, double *M, double *h, double rho, double alpha, double *old_h)
{
  alpha /= rho;
  
  int len;
  int *sumlen;
  double *mysum;
  double *ans;
  struct RANK_SCORE *myM;
  
  sumlen = new int[n+2];
  mysum = new double[n+2];
  ans = new double[n+2];
  myM = new RANK_SCORE[n+2];
  
  sumlen[0] = 0;
  mysum[0] = 0;
  len = 0;

  for (int i=1; i<id; ++i)
    {
      myM[i].value = M[i];
      myM[i].index = i;
    }

  for (int i=id+1; i<n+1; ++i)
    {
      myM[i-1].value = M[i];
      myM[i-1].index = i;
    }
  
  sort(myM+1,myM+n,cmp_absolute_score);
  
  for (int i=1; i<n; ++i)
    {
      mysum[i] = mysum[i-1] + fabs(myM[i].value)-h[i]*alpha;
      int left = 1;
      int right = len;
      int mid;

      while (right >= left)
        {
          mid = (left+right)/2;

          double tmp = ans[mid]*(i-sumlen[mid-1])-(-mysum[sumlen[mid-1]]+mysum[i]);

          if (tmp == 0 || ans[mid] == 0)
            {
              break;
            }
          else if (tmp > 0)
            {
              left = mid+1;
            }
          else
            {
              right = mid-1;
            }
        }

      if (left > right)
        {
          len = right+1;
          sumlen[len] = i;
          ans[len] = (mysum[i]-mysum[sumlen[len-1]]>0)?(mysum[i]-mysum[sumlen[len-1]]):0;
          if ((i-sumlen[len-1])!=0)
            ans[len] /= (i-sumlen[len-1]);
         
        }
      else
        {
          len = mid;
          ans[mid] = (mysum[i]-mysum[sumlen[mid-1]]>0)?(mysum[i]-mysum[sumlen[mid-1]]):0;
          sumlen[mid] = i;
          if ((i-sumlen[mid-1])!=0)
            ans[mid] /= (i-sumlen[mid-1]);
        }
    }
  
  for (int i=1; i<=len; ++i)
    {
      for (int j=sumlen[i-1]+1; j<=sumlen[i]; ++j)
        {
          if (myM[j].value > 0)
            {
              myans[myM[j].index] = ans[i];
            }
          else
            {
              myans[myM[j].index] = 0-ans[i];
            }
        }
    }
  
  myans[id] = M[id];
  
  delete []sumlen;
  delete []mysum;
  delete []myM;
  delete []ans;
}

void NON_SYMMETRIC_OPT(int n, double **myans, double **M, double *h, double *th, double *g, double rho, double alpha, double beta)
{ 
  double **tmph;
  tmph = new double *[n+1];
  for (int i=0; i<n+1; ++i)
    { 
      tmph[i] = new double [n+1];
    }

  for (int i=1; i<n+1; ++i)
    { 
      for (int j=1; j<n; ++j)
        { 
          tmph[i][j] = alpha*h[j] + beta*g[i]*th[j];
        }

      ROW_SCORE(n,i,myans[i],M[i],tmph[i],rho,1,th);
    }

  for (int i=0; i<n+1; ++i)
    { 
      delete [] tmph[i];
    }
  delete []tmph;
}

void DD(int n, double **myans, double **M, double *ph, double *h, double *g, double rho, double alpha, double theta)
{
  int **check_sign;
  double **step_size;
  double **beta;
  bool break_sign;
  
  check_sign = new int *[n+1];
  step_size = new double *[n+1];
  beta = new double *[n+1];
  
  srand(time(NULL));
  
  for (int i=0; i<n+1; ++i)
    {
      beta[i] = new double [n+1];
      step_size[i] = new double [n+1];
      check_sign[i] = new int [n+1];
    }
  
  for (int i=0; i<n+1; ++i)
    {
      for (int j=0; j<n+1; ++j)
        {
          beta[i][j] = M[i][j];
          step_size[i][j] = DD_STEPSIZE;
          check_sign[i][j] = 0;
        }
    }

  for (int i=0; i<DD_ITER; ++i)
    {
      break_sign = true;

      NON_SYMMETRIC_OPT(n, myans, beta, ph, h, g, rho, alpha, theta);

      for (int j=1; j<n+1; ++j)
        {
          for (int k=j; k<n+1; ++k)
            {
              if (fabs(myans[j][k]-myans[k][j]) > DD_EPS*(fabs(myans[j][k])+fabs(myans[k][j])))
                {
                  break_sign = false;
                }

              if ((myans[j][k]-myans[k][j])*check_sign[j][k] > 0)
                {
                  step_size[j][k] = step_size[j][k]*DD_CHANGE*(1+rand()%100/100.0);
                }
              else if ((myans[j][k]-myans[k][j])*check_sign[j][k] < 0)
                {
                  step_size[j][k] = step_size[j][k]/(DD_CHANGE*(1+rand()%100/100.0));
                }
              if (myans[j][k] - myans[k][j] > 0)
                { 
                  check_sign[j][k] = 1;
                }
              else if (myans[j][k] - myans[k][j] < 0)
                {
                  check_sign[j][k] = -1;
                }
              else
                {
                  check_sign[j][k] = 0;
                }

              beta[j][k] -= 2*step_size[j][k]/rho*(myans[j][k]-myans[k][j]);
              beta[k][j] += 2*step_size[j][k]/rho*(myans[j][k]-myans[k][j]);
            }
        }

      if (break_sign)
        {
          break;
        }
    }

  for (int i=1; i<n+1; ++i)
    {
      for (int j=i; j<n+1; ++j)
        {
          myans[i][j] = myans[j][i] = (myans[i][j]+myans[j][i])/2;
        }
    }
  
  for (int i=0; i<n+1; ++i)
    {
      delete [] check_sign[i];
      delete [] step_size[i];
      delete [] beta[i];
    }
  delete []check_sign;
  delete []step_size;
  delete []beta;
}

void ADJUST(double **myans, double **M, int p, double *h, double *th, double *g, double rho, double theta)
{
  double *penaltyg;
  penaltyg = new double [p+1];

  for (int i=1; i<p+1; ++i)
    {
      penaltyg[i] = g[i];
    }

  DD(p, myans, M, h, th, penaltyg, rho, 0, theta);

  delete []penaltyg;
}



void Lovasz(double *M, double *Z, double *expected, double *observed, int p, double rho, double beta,double *ans,double ScaleFree)
{
  // All the inputs are vectors
  // Ans is a 1 by (p*p) vector
  // M is a 1 by (p*p) vector
  // Z is a 1 by (p*p) vector
  // rank is a 1 by p vector

  double *h;
  int *g;
  double *tg;
  double *th;

  double **re_M;
  double **re_ans;
  double *re_observed;
  double *re_expected;

  re_M = new double*[p+1];
  re_ans = new double*[p+1];
  re_observed = new double[p+1];
  re_expected = new double[p+1];
  for (int i=0; i<p+1; ++i)
    {
      re_ans[i] = new double[p+1];
      re_M[i] = new double[p+1];
    }
  
  h = new double[p+1];
  g = new int[p+1];
  th = new double[p+1];
  tg = new double[p+1];

  for (int i=0; i<p+1; ++i)
    {
      re_M[i][0] = 0;
      re_M[0][i] = 0;
      re_ans[i][0] = 0;
      re_ans[i][0] = 0;
      
    }

  for (int i=0; i<p; ++i)
    {
      for (int j=0; j<p; ++j)
	{
	  re_M[i+1][j+1] = M[i*p+j]+Z[i*p+j];
      if (re_M[i+1][j+1]<0)
          re_M[i+1][j+1]=0;
	}
      re_observed[i+1] = observed[i];
      re_expected[i+1] = expected[i];
    }
  
  GENH(p,h);
  GENPP(p,re_expected,re_observed,g);
  TH(p,th,ScaleFree);
  TG(p,g,th,tg);

  ADJUST(re_ans, re_M, p, h, th, tg, rho, beta);

  for (int i=0; i<p; ++i)
    {
      for (int j=0; j<p; ++j)
	{
	  ans[i*p+j] = re_ans[i+1][j+1];
	}
    }

  // Release memory
  for (int i=0; i<p+1; ++i)
    {
      delete [] re_M[i];
      delete [] re_ans[i];
    }

  delete []re_M;
  delete []re_ans;
  delete []re_expected;
  delete []re_observed; 
  delete []h;
  delete []g;
  delete []tg;
  delete []th;
}

void mexFunction ( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] ){
	double *Matrix;
    double *ans;
    double *Z;

    double *expected;
    double rho;
    double beta;
    int n;
    double *observed;
    double ScaleFree;
 
	Matrix = mxGetPr(prhs[0]);
    Z=mxGetPr(prhs[1]);
	expected = mxGetPr(prhs[2]);
    observed=mxGetPr(prhs[3]);
	n = mxGetScalar ( prhs[4] );
  
	rho = mxGetScalar ( prhs[5] );
	beta = mxGetScalar ( prhs[6] );
    
	plhs[0] = mxCreateDoubleMatrix ( n*n,1, mxREAL );
	ans= mxGetPr ( plhs[0] );
    ScaleFree= mxGetScalar ( prhs[7] );
	Lovasz(Matrix, Z, expected, observed, n, rho, beta,ans,ScaleFree);
	return;

} 

